package InvertedPendulum "Example of a linear quadratic regulator used to stabilize an inverted pendulum"
  package Components "Components package for the InvertedPendulum example"
    extends Modelica.Icons.Package;

    partial model BasePendulumModel "Partial inverted pendelum model"
      PlanarMechanics.Parts.Body pendulum(I = pendulumInertia, m = pendulumMass) "Pendulum mass." annotation(Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {-50, -80}), visible = true));
      PlanarMechanics.Parts.Fixed origin "Center point. Reference point for cart." annotation(Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-100, 10}), visible = true));
      PlanarMechanics.Parts.Body cart(m = cartMass, I = cartInertia) "Mass of cart." annotation(Placement(transformation(extent = {{0, 20}, {20, 40}}, origin = {-40, -20}, rotation = 0), visible = true));
      inner PlanarMechanics.PlanarWorld planarWorld(animateGravity = false) annotation(Placement(transformation(extent = {{-80, -20}, {-60, 0}}, origin = {-20, -60}, rotation = 0), visible = true));
      PlanarMechanics.Joints.Revolute revolute(useFlange = true, phi.start = angle0, phi.fixed = true) "Pendulum axis of rotation." annotation(Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = -270, origin = {-50, -20}), visible = true));
      PlanarMechanics.Joints.Prismatic cartPosition(useFlange = true, r = {1, 0}, s(fixed = true, start = 0), v(fixed = true), animate = false) "Translational position of cart." annotation(Placement(transformation(extent = {{-38, 20}, {-18, 40}}, origin = {-42, -20}, rotation = 0), visible = true));
      Modelica.Mechanics.Rotational.Sensors.AngleSensor angle "Angle sensor for the pendulum." annotation(Placement(visible = true, transformation(origin = {20, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Blocks.Math.MatrixGain controller(K = {{-59.8109, -15.9831, 1., 3.03214}}) "LQ-regulator with optimal gains." annotation(Placement(visible = true, transformation(origin = {60, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sensors.PositionSensor position "Position sensor for the cart." annotation(Placement(visible = true, transformation(origin = {20, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sensors.SpeedSensor angularVelocity "Velocity sensor for the pendulum." annotation(Placement(visible = true, transformation(origin = {20, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sensors.SpeedSensor speed "Velocity sensor for the cart." annotation(Placement(visible = true, transformation(origin = {20, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      PlanarMechanics.Sources.WorldForce disturbancePendulum(N_to_m = 0.1, color = {163, 50, 79}) "Disturbance force applied on the pendulum." annotation(Placement(visible = true, transformation(origin = {-10, -70}, extent = {{10, -10}, {-10, 10}}, rotation = -360)));
      PlanarMechanics.Sources.WorldForce disturbanceCart(N_to_m = 0.1, color = {27, 89, 174}) "Disturbance force applied on the cart." annotation(Placement(visible = true, transformation(origin = {-10, -50}, extent = {{10, -10}, {-10, 10}}, rotation = -360)));
      constant Modelica.Units.SI.Length widthUnitLength = 1;
      PlanarMechanics.Parts.FixedTranslation pendulumLength(r = {0, length}, width = widthUnitLength / planarWorld.defaultWidthFraction) "Length of the pendulum." annotation(Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {-50, -50}), visible = true));
      PlanarMechanics.Sources.WorldForce controllerForce(N_to_m = 0.1, color = {187, 160, 49}) "Corrective force applied to the cart." annotation(Placement(visible = true, transformation(origin = {-30, 70}, extent = {{10, -10}, {-10, 10}}, rotation = -360)));
      parameter Modelica.Units.SI.Mass cartMass = 1 "Mass of the cart (base of the pendulum)" annotation(Dialog(group = "Pendulum Properties"));
      parameter Modelica.Units.SI.Inertia cartInertia = 0.1 "Inertia of the cart (base of the pendulum)" annotation(Dialog(group = "Pendulum Properties"));
      parameter Modelica.Units.SI.Mass pendulumMass = 0.5 "Mass at the top of the pendulum" annotation(Dialog(group = "Pendulum Properties"));
      parameter Modelica.Units.SI.Inertia pendulumInertia = 0.1 "Inertia at the top of the pendulum" annotation(Dialog(group = "Pendulum Properties"));
      parameter Modelica.Units.SI.Angle angle0 = 0 "Initial angle of the pendulum" annotation(Dialog(group = "Pendulum Properties"));
      constant Modelica.Units.SI.Length length = 1 "Distance between the cart and the top of the pendulum" annotation(Dialog(group = "Pendulum Properties"));
      Modelica.Units.SI.Position pendelum_pos[2] = pendulum.r "transl. position (pendulum.r)";
      Modelica.Blocks.Interfaces.RealOutput s = position.s "Absolute position of flange as output signal (position.s)";
      Modelica.Blocks.Interfaces.RealOutput v = speed.v "Absolute velocity of flange as output signal (speed.v)";
      Modelica.Blocks.Interfaces.RealOutput phi = angle.phi "Absolute angle of flange as output signal (angle.phi)";
      Modelica.Blocks.Interfaces.RealOutput w = angularVelocity.w "Absolute angular velocity of flange as output signal (angularVelocity.w)";
      Modelica.Blocks.Interfaces.RealOutput y[controller.nout] = controller.y "Connector of Real output signals (controller.y)";
      Real cartDisturbance[3] = disturbanceCart.force "x-, y-coordinates of force and torque resolved in world frame (disturbanceCart.force)";
      Real pendelumDisturbance[3] = disturbancePendulum.force "x-, y-coordinates of force and torque resolved in world frame (disturbancePendulum.force)";
      Modelica.Units.SI.Acceleration a = cartPosition.a "acceleration of elongation (cartPosition.a)";
      Modelica.Units.SI.AngularAcceleration z = revolute.z "Angular acceleration (revolute.z)";
    equation
      connect(revolute.frame_a, cart.frame_a) annotation(Line(points = {{-3.333, -13.333}, {-3.333, 6.667}, {6.667, 6.667}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {-46.667, 3.333}));
      connect(origin.frame, cartPosition.frame_a) annotation(Line(points = {{-5, 0}, {5, 0}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {-85, 10}));
      connect(cartPosition.frame_b, cart.frame_a) annotation(Line(points = {{-10, 0}, {10, 0}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {-50, 10}));
      connect(position.flange, cartPosition.flange_a) annotation(Line(visible = true, origin = {-43.333, 39.667}, points = {{53.333, 10.333}, {-26.667, 10.333}, {-26.667, -20.667}}, color = {0, 127, 0}));
      connect(revolute.flange_a, angle.flange) annotation(Line(visible = true, origin = {-2.5, -15}, points = {{-37.5, -5}, {2.5, -5}, {2.5, 5}, {12.5, 5}}, color = {64, 64, 64}));
      connect(revolute.flange_a, angularVelocity.flange) annotation(Line(visible = true, origin = {-2.5, -25}, points = {{-37.5, 5}, {2.5, 5}, {2.5, -5}, {12.5, -5}}, color = {64, 64, 64}));
      connect(cartPosition.flange_a, speed.flange) annotation(Line(visible = true, origin = {-43.333, 26.333}, points = {{-26.667, -7.333}, {-26.667, 3.667}, {53.333, 3.667}}, color = {0, 127, 0}));
      connect(position.s, controller.u[3]) annotation(Line(visible = true, origin = {42.25, 30}, points = {{-11.25, 20}, {-2.25, 20}, {-2.25, -20}, {5.75, -20}}, color = {1, 37, 163}));
      connect(speed.v, controller.u[4]) annotation(Line(visible = true, origin = {42.25, 20}, points = {{-11.25, 10}, {-2.25, 10}, {-2.25, -10}, {5.75, -10}}, color = {1, 37, 163}));
      connect(angle.phi, controller.u[1]) annotation(Line(visible = true, origin = {42.25, 0}, points = {{-11.25, -10}, {-2.25, -10}, {-2.25, 10}, {5.75, 10}}, color = {1, 37, 163}));
      connect(angularVelocity.w, controller.u[2]) annotation(Line(visible = true, origin = {42.25, -10}, points = {{-11.25, -20}, {-2.25, -20}, {-2.25, 20}, {5.75, 20}}, color = {1, 37, 163}));
      connect(pendulumLength.frame_b, pendulum.frame_a) annotation(Line(points = {{0, 5}, {0, -5}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {-50, -65}));
      connect(pendulumLength.frame_a, revolute.frame_b) annotation(Line(visible = true, origin = {-50, -30}, points = {{0, -10}, {0, 0}, {0, 0}}, color = {95, 95, 95}));
      connect(disturbancePendulum.frame_b, pendulum.frame_a) annotation(Line(visible = true, origin = {-32.5, -65}, points = {{12.5, -5}, {-17.5, -5}}, color = {95, 95, 95}));
      connect(disturbanceCart.frame_b, cart.frame_a) annotation(Line(visible = true, origin = {-36.667, -15}, points = {{16.667, -35}, {6.667, -35}, {6.667, 10}, {-13.333, 10}, {-13.333, 25}, {-3.333, 25}}, color = {95, 95, 95}));
      connect(controllerForce.frame_b, cart.frame_a) annotation(Line(visible = true, origin = {-45, 40}, points = {{5, 30}, {-5, 30}, {-5, -30}, {5, -30}}, color = {95, 95, 95}));
      connect(controllerForce.force[1], controller.y[1]) annotation(Line(visible = true, origin = {53.25, 40}, points = {{-71.25, 30}, {26.75, 30}, {26.75, -30}, {17.75, -30}}, color = {0, 36, 164}));
      annotation(Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Ellipse(visible = true, lineColor = {75, 138, 73}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-100, -100}, {100, 100}}), Polygon(visible = true, lineColor = {0, 0, 255}, fillColor = {75, 138, 73}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-36, 60}, {64, 0}, {-36, -60}, {-36, 60}})}), Documentation(info = "<html>
    <p>
This model is a partial model containing the dynamics of the inverted pendelum.
</p></html>", figures = {Figure(title = "Controller inputs", identifier = "94c7c", preferred = true, plots = {Plot(curves = {Curve(y = s, legend = "Cart Position"), Curve(y = v, legend = "Cart Velocity"), Curve(y = phi, legend = "Pendulum Angle"), Curve(y = w, legend = "Pendulum Anglular Velocity")})}), Figure(title = "Controller output", identifier = "b69d7", plots = {Plot(curves = {Curve(y = y[1], legend = "Controller Output")}, y = Axis(label = ""))}), Figure(title = "Cart states", identifier = "c5f0e", plots = {Plot(curves = {Curve(y = s, legend = "Cart Position"), Curve(y = v, legend = "Cart Velocity"), Curve(y = a, legend = "Cart Acceleration")})}), Figure(title = "Pendulum states", identifier = "58344", plots = {Plot(curves = {Curve(y = phi, legend = "Pendulum Angle"), Curve(y = w, legend = "Pendulum Anglular Velocity"), Curve(y = z, legend = "Pendulum Anglular Acceleration")})}), Figure(title = "Pendulum X-Y position", identifier = "2887e", plots = {Plot(curves = {Curve(x = pendelum_pos[1], y = pendelum_pos[2])}, x = Axis(label = "X"), y = Axis(label = "Y"))}), Figure(title = "Disturbance forces", identifier = "8a232", plots = {Plot(curves = {Curve(y = cartDisturbance[1], legend = "x-disturbance on cart"), Curve(y = cartDisturbance[2], legend = "y-disturbance on cart"), Curve(y = cartDisturbance[3], legend = "Torque-disturbance on cart")}, y = Axis(label = "[N]/[N.m]")), Plot(curves = {Curve(y = pendelumDisturbance[1], legend = "x-disturbance on pendulum"), Curve(y = pendelumDisturbance[2], legend = "y-disturbance on pendulum"), Curve(y = pendelumDisturbance[3], legend = "Torque-disturbance on pendulum")}, x = Axis(min = 0, max = 20), y = Axis(label = "[N]/[N.m]"))}), Figure(title = "Corrective force", identifier = "2a222", plots = {Plot(curves = {Curve(y = controllerForce.force[1], legend = "Corrective force on cart, along x-axis")}, y = Axis(label = "[N]"))})}), __Wolfram, Diagram(coordinateSystem(extent = {{-120, -105}, {100, 90}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
    end BasePendulumModel;
    annotation(Documentation, Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end Components;

  model InvertedPendulumPulse "A LQR-controlled inverted pendulum system"
    extends Components.BasePendulumModel;
    parameter Modelica.Units.SI.Force cartDisturbanceForce = 0.1 "Ampliude of disturbance force applied to cart" annotation(Dialog(group = "Disturbance Properties"));
    parameter Modelica.Units.SI.Force pendulumDisturbanceForce = 1.5 "Ampliude of disturbance force applied to the top of the pendulum" annotation(Dialog(group = "Disturbance Properties"));
    parameter Modelica.Units.SI.Time cartDisturbancePeriod = 4 "Period of the disturbance pulse on the cart" annotation(Dialog(group = "Disturbance Properties"));
    parameter Modelica.Units.SI.Time pendulumDisturbancePeriod = 1 "Period of the disturbance pulse on the top of the pendulum" annotation(Dialog(group = "Disturbance Properties"));
    Modelica.Blocks.Sources.Pulse disturbanceCartPulse(amplitude = cartDisturbanceForce, period = cartDisturbancePeriod, nperiod = 5, startTime = 4) "Magnitude of disturbance force applied on the cart." annotation(Placement(visible = true, transformation(origin = {90, -30}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
    Modelica.Blocks.Sources.Pulse disturbancePendulumPulse(amplitude = pendulumDisturbanceForce, width = 100, period = pendulumDisturbancePeriod, nperiod = 1, startTime = 6) "Magnitude of disturbance force applied on the pendulum." annotation(Placement(visible = true, transformation(origin = {90, -70}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
  equation
    disturbancePendulum.force[2:3] = {0, 0};
    disturbanceCart.force[2:3] = {0, 0};
    controllerForce.force[2:3] = {0, 0};
    connect(disturbanceCartPulse.y, disturbanceCart.force[1]) annotation(Line(visible = true, origin = {56.858, -40}, points = {{22.142, 10}, {3.142, 10}, {3.142, -10}, {-54.858, -10}}, color = {1, 37, 163}));
    connect(disturbancePendulumPulse.y, disturbancePendulum.force[1]) annotation(Line(visible = true, origin = {40.5, -70}, points = {{38.5, 0}, {-38.5, 0}}, color = {1, 37, 163}));
    annotation(preferredView = "info", __Wolfram(ControlPanels(Panel(identifier = "reverent-gödel", title = "Model parameters", elements = {InputField(variable = angle0, label = "Initial angle of the pendulum"), InputField(variable = cartDisturbanceForce, label = "Ampliude of disturbance force applied to cart."), InputField(variable = cartDisturbancePeriod, label = "Period of the disturbance pulse on the cart."), InputField(variable = cartInertia, label = "Inertia of the cart (base of the pendulum)"), InputField(variable = cartMass, label = "Mass of the cart (base of the pendulum)"), InputField(variable = pendulumDisturbanceForce, label = "Ampliude of disturbance force applied to the top of the pendulum"), InputField(variable = pendulumDisturbancePeriod, label = "Period of the disturbance pulse on the top of the pendulum"), InputField(variable = pendulumInertia, label = "Inertia at the top of the pendulum"), InputField(variable = pendulumMass, label = "Mass at the top of the pendulum")}))), Documentation(info = "<html>
<div style=\"background-color: #f4fbff; padding: 1rem; border: 2px solid #d9e5f3\">

<h2>Library Dependency</h2>
   <p class=\"\">
		This model requires the <a href=\"https://www.wolfram.com/system-modeler/libraries/model-plug/\">ModelPlug</a> and <a href=\"https://www.wolfram.com/system-modeler/libraries/planar-mechanics/\">PlanarMechanics</a> libraries.
	</p>
	<ul>
		<li style=\"background: url('modelica://EducationExamples/Resources/ModelPlug.png') no-repeat left top; background-size: 32px 32px; padding-left: 40px; margin-bottom: 15px; \">ModelPlug is a free Modelica library that allows you to link your simulation models to the real world by the use of an <a href=\"https://www.arduino.cc/\">Arduino Board</a> or any other board following the <a href=\"http://www.firmata.org/\">Firmata</a> protocol). ModelPlug opens up a wide variety of ways to interact with your models, for example by using buttons, switches, input sensor information or even actuators such as motors and servos.</li>
<li style=\"background: url('modelica://EducationExamples/Resources/PlanarMechanics.png') no-repeat left top; background-size: 32px 32px; padding-left: 40px; \">The free PlanarMechanics library was created especially for modeling multibody systems with two-dimensional mechanical components. Compared to the MultiBody library, currently available in the Modelica Standard Library, it is simpler to use and it is more optimized to planar modeling. Planar models of mechanical systems are useful in many different applications, for example, in contact problems that are more easily modeled in 2D than in 3D.</li>

	</ul>

</div>

    <h4>Model</h4>
    <p>
The inverted pendulum model consists of a pendulum and a cart, with the axis of rotation of the pendulum being located at the center of the cart. The pendulum is initialized with its center of mass above the rotation axis. When located directly above the cart, the pendulum will be in steady state and will stay there until disturbed.
</p><p>
While a pendulum that hangs straight down will be in a stable position, an inverted pendulum is unstable. That means that any small disturbance will cause the pendulum to tip over and never return to its original position.
</p><p>

<p style=\"text-align: center;\"><img src=\"modelica://InvertedPendulum/DocumentationFiles/pendulum.png\" style=\"width:400px\" alt=\"\" /></p>
    <h4>Control System</h4>
    <p>
A control system can be used to stabilize the unstable pendulum. Here, a linear-quadratic regulator is used. First, the regulator measures the states of the system, namely the cart position and velocity, the pendulum angle and the angular velocity. The regulator then calculates a force that should be applied on the cart in order for all the states to become zero. In other words, the regulator aims to have the pendulum pointing straight up (defined here as 0 degrees) and the cart return to the origin.
</p>

<p style=\"text-align: center;\"><img src=\"modelica://InvertedPendulum/DocumentationFiles/controlSystem.png\" alt=\"controlSystem\" style=\"width:391.5px\"  alt=\"\" /></p>
    <h4>Simulation</h4>
    <p>
By simulating the model, you can try different values of the parameters and see how the control system responds.
</p><p>
To simulate the model and see the generated 3D animation, follow the steps below:
</p>
Click the Simulate button: <img src=\"modelica://InvertedPendulum/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"simulate\"/></li>
<p>When the simulation is finished, click the Animate button: <img src=\"modelica://InvertedPendulum/DocumentationFiles/animate.svg\" style=\"width:20px\" alt=\"animate\"/> </p>
<p>Use your mouse or trackpad to drag the animation into a good angle and zoom in with your scroll wheel or by using the trackpad. Then click the Play button to play the animation: <img src=\"modelica://InvertedPendulum/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"simPlay\"/>
</p>

<p>
You can change the magnitude of the force that is applied to the pendulum by changing the <span>pendulumDisturbanceForce</span> parameter.
</p>


</html>"), experiment(StopTime = 20, __Wolfram_SynchronizeWithRealTime = false), Diagram(coordinateSystem(extent = {{-120, -105}, {120, 90}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end InvertedPendulumPulse;

  model InvertedPendulumHardware "A LQR-controlled inverted pendulum system, controlled by external hardware using the ModelPlug library"
    extends Components.BasePendulumModel;
    ModelPlug.Pins.AnalogInput cartInput(Pin = cartPin, MinValue = cartMinValue, MaxValue = cartMaxValue) annotation(Placement(visible = true, transformation(origin = {110, -30}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
    ModelPlug.Boards.Arduino arduino(Port = port) annotation(Placement(visible = true, transformation(origin = {130, 10}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
    ModelPlug.Pins.AnalogInput pendulumInput(Pin = pendulumPin, MinValue = pendulumMinValue, MaxValue = pendulumMaxValue) annotation(Placement(visible = true, transformation(origin = {110, -70}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
    parameter ModelPlug.Internal.Types.SerialPort port = "COM4" "Name of the serial port" annotation(Dialog(group = "Arduino Parameters"));
    parameter Integer cartPin = 14 "Number of the analog pin" annotation(Dialog(group = "Arduino Parameters"));
    parameter Real cartMinValue = -0.22 "Minimum value when the ADC reads 0" annotation(Dialog(group = "Arduino Parameters"));
    parameter Real cartMaxValue = 0.5 "Maximum value when the ADC reads 1024" annotation(Dialog(group = "Arduino Parameters"));
    parameter Integer pendulumPin = 15 "Number of the analog pin" annotation(Dialog(group = "Arduino Parameters"));
    parameter Real pendulumMinValue = -0.22 "Minimum value when the ADC reads 0" annotation(Dialog(group = "Arduino Parameters"));
    parameter Real pendulumMaxValue = 0.5 "Maximum value when the ADC reads 1024" annotation(Dialog(group = "Arduino Parameters"));
  equation
    disturbancePendulum.force[2:3] = {0, 0};
    disturbanceCart.force[2:3] = {0, 0};
    controllerForce.force[2:3] = {0, 0};
    connect(cartInput.y, disturbanceCart.force[1]) annotation(Line(visible = true, origin = {73, -40}, points = {{27, 10}, {22, 10}, {22, -10}, {-71, -10}}, color = {1, 37, 163}));
    connect(pendulumInput.y, disturbancePendulum.force[1]) annotation(Line(visible = true, origin = {51, -70}, points = {{49, 0}, {-49, 0}}, color = {1, 37, 163}));
    connect(cartInput.pinConnector, arduino.boardConnector) annotation(Line(visible = true, origin = {126.667, -16.667}, points = {{-6.667, -13.333}, {3.333, -13.333}, {3.333, 26.667}}));
    connect(pendulumInput.pinConnector, arduino.boardConnector) annotation(Line(visible = true, origin = {126.667, -43.333}, points = {{-6.667, -26.667}, {3.333, -26.667}, {3.333, 53.333}}));
    annotation(preferredView = "info", Documentation(info = "<html>


<div style=\"background-color: #f4fbff; padding: 1rem; border: 2px solid #d9e5f3\">

<h2>Library Dependency</h2>
   <p class=\"\">
		This model requires the <a href=\"https://www.wolfram.com/system-modeler/libraries/model-plug/\">ModelPlug</a> and <a href=\"https://www.wolfram.com/system-modeler/libraries/planar-mechanics/\">PlanarMechanics</a> libraries.
	</p>
	<ul>
		<li style=\"background: url('modelica://EducationExamples/Resources/ModelPlug.png') no-repeat left top; background-size: 32px 32px; padding-left: 40px; margin-bottom: 15px; \">ModelPlug is a free Modelica library that allows you to link your simulation models to the real world by the use of an <a href=\"https://www.arduino.cc/\">Arduino Board</a> or any other board following the <a href=\"http://www.firmata.org/\">Firmata</a> protocol). ModelPlug opens up a wide variety of ways to interact with your models, for example by using buttons, switches, input sensor information or even actuators such as motors and servos.</li>
<li style=\"background: url('modelica://EducationExamples/Resources/PlanarMechanics.png') no-repeat left top; background-size: 32px 32px; padding-left: 40px; \">The free PlanarMechanics library was created especially for modeling multibody systems with two-dimensional mechanical components. Compared to the MultiBody library, currently available in the Modelica Standard Library, it is simpler to use and it is more optimized to planar modeling. Planar models of mechanical systems are useful in many different applications, for example, in contact problems that are more easily modeled in 2D than in 3D.</li>

	</ul>

</div>



    <h4>Model</h4>

    <p>
This model is a variation of the <a href=\"modelica://InvertedPendulum.InvertedPendulumPulse#diagram\">main example model</a>. It uses connection to external hardware to control the magnitude of the disturbance forces.
</p>
    <h4>External Connection</h4>
    <p>
To run this model, you will need to connect an Arduino board to your computer and control the input to two analog pins, for example, thumbwheel potentiometers. Adjust the port of the Arduino component to the one used by your connected board with the <span>Port</span> parameter.
</p>

</html>"), experiment(StopTime = -1, Interval = 0.1, __Wolfram_SynchronizeWithRealTime = true), __Wolfram(PlotSet(plots = {Plot(name = "ControllerInputs", preferred = true, subPlots = {SubPlot(curves = {Curve(x = time, y = controller.u[1], legend = "Angle"), Curve(x = time, y = controller.u[2], legend = "Angular Velocity"), Curve(x = time, y = controller.u[3], legend = "Position"), Curve(x = time, y = controller.u[4], legend = "Speed")})}), Plot(name = "XY-Position", subPlots = {SubPlot(curves = {Curve(x = pendulum.r[1], y = pendulum.r[2])})}), Plot(name = "PositionVelocityAcceleration", subPlots = {SubPlot(curves = {Curve(x = time, y = cartPosition.s, legend = "Position [m]"), Curve(x = time, y = cartPosition.v, legend = "Velocity [m/s]"), Curve(x = time, y = cartPosition.a, legend = "Acceleration [m/s²]")})}), Plot(name = "AppliedForces", subPlots = {SubPlot(curves = {Curve(x = time, y = controllerForce.force[1], legend = "Controller Force"), Curve(x = time, y = disturbanceCart.force[1], legend = "Cart Distrurbance Force"), Curve(x = time, y = disturbancePendulum.force[1], legend = "Pendulum Disturbance Force")})})})), Diagram(coordinateSystem(extent = {{-120, -105}, {148.5, 90}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end InvertedPendulumHardware;
  annotation(Documentation(info = "<div style=\"background-color: #f4fbff; padding: 1rem; border: 2px solid #d9e5f3\">

<h2>Library Dependency</h2>
   <p class=\"\">
		This model requires the <a href=\"https://www.wolfram.com/system-modeler/libraries/model-plug/\">ModelPlug</a> and <a href=\"https://www.wolfram.com/system-modeler/libraries/planar-mechanics/\">PlanarMechanics</a> libraries.
	</p>
	<ul>
		<li style=\"background: url('modelica://EducationExamples/Resources/ModelPlug.png') no-repeat left top; background-size: 32px 32px; padding-left: 40px; margin-bottom: 15px; \">ModelPlug is a free Modelica library that allows you to link your simulation models to the real world by the use of an <a href=\"https://www.arduino.cc/\">Arduino Board</a> or any other board following the <a href=\"http://www.firmata.org/\">Firmata</a> protocol). ModelPlug opens up a wide variety of ways to interact with your models, for example by using buttons, switches, input sensor information or even actuators such as motors and servos.</li>
<li style=\"background: url('modelica://EducationExamples/Resources/PlanarMechanics.png') no-repeat left top; background-size: 32px 32px; padding-left: 40px; \">The free PlanarMechanics library was created especially for modeling multibody systems with two-dimensional mechanical components. Compared to the MultiBody library, currently available in the Modelica Standard Library, it is simpler to use and it is more optimized to planar modeling. Planar models of mechanical systems are useful in many different applications, for example, in contact problems that are more easily modeled in 2D than in 3D.</li>

	</ul>

</div>"), preferredView = "info", uses(Modelica(version = "4.0.0"), PlanarMechanics(version = "1.6.0"), ModelPlug(version = "2.0")), __Wolfram(openClass = InvertedPendulum.InvertedPendulumPulse), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90, -90}, {90, 90}}, radius = 25), Text(visible = true, origin = {12, 20.833}, textColor = {255, 255, 255}, extent = {{-102, -70.833}, {78, 29.167}}, textString = "Ex", fontName = "Arial")}), Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
end InvertedPendulum;
